﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using DataAPI.Models;
using Newtonsoft.Json.Linq;

namespace DataAPIClient
{
    public partial class frmTestECMInfo : Form
    {
        public string TestURL;
        public string TestAuth;

        // exit out of the get loop when tru
        private bool AbortFlag = false;
        const string LONG_DATE_STRING = "yyyy-MM-dd HH:mm:ss";
        const string ISO_DATE_STRING = "yyyy-MM-ddTHH:mm:ssZ";

        public frmTestECMInfo()
        {
            InitializeComponent();
        }

        /// <summary>
        /// return a ListViewItem 
        /// </summary>
        /// <param name="v">ECMInfo object filled via the web service</param>
        /// <returns>new ListViewItem with ECMInfo in the correct columns</returns>
        private ListViewItem ECMListViewItem(ECMInfo e)
        {
            ListViewItem lvi = new ListViewItem(e.Id.ToString());
            lvi.SubItems.Add(e.UTCDate.ToString(LONG_DATE_STRING));
            lvi.SubItems.Add(e.DevId.ToString());
            lvi.SubItems.Add(e.Lat.ToString("N6"));
            lvi.SubItems.Add(e.Lon.ToString("N6"));
            lvi.SubItems.Add(e.ValCool ? "yes" : "no");
            lvi.SubItems.Add(e.AvgCool.ToString("N1"));
            lvi.SubItems.Add(e.ValRPM ? "yes" : "no");
            lvi.SubItems.Add(e.AvgRPM.ToString("N1"));
            lvi.SubItems.Add(e.Odom.ToString("N1"));
            lvi.SubItems.Add(e.EngHrs.ToString("N1"));
            lvi.SubItems.Add(e.IdleTim.ToString("N1"));
            
            lvi.Tag = e;        // save for later reference
            return lvi;
        }

        /// <summary>
        /// do most of the work in here
        /// perform the Get and deserialize the JSon results into a ECMInfo array
        /// </summary>
        /// <param name="requestString">the Get request string</param>
        /// <returns>ECMInfo array</returns>
        private ECMInfo[] PerformECMInfoGet(string requestString)
        {
            ECMInfo[] Response = new ECMInfo[0];
            Uri baseUri = new Uri(frmMain.FixUri(TestURL));
            // combine the uri's
            Uri requestUri = new Uri(baseUri, requestString);
            // and make a web request
            HttpWebRequest ecmInfoRequest = (HttpWebRequest)WebRequest.Create(requestUri);
            // authorization, looks up 
            ecmInfoRequest.Headers.Add("Authorization", Uri.EscapeDataString(TestAuth));
            ecmInfoRequest.Method = "Get";
            // actually perform the Get
            WebResponse ecmInfoResponse = ecmInfoRequest.GetResponse();
            using (ecmInfoResponse)
            {
                // convert the Json stream to JArray then to ECMInfo array
                using (Stream stream = (Stream)ecmInfoResponse.GetResponseStream())
                {
                    string jsonString = new StreamReader(stream).ReadToEnd();
                    JArray jarray = JArray.Parse(jsonString);
                    // is it the end?
                    if (jarray.IsNullOrEmpty())
                        Response = null;
                    else
                        Response = jarray.ToObject<ECMInfo[]>();
                }
                // make sure it's valid
                if (Response != null)
                {
                    // if we've got something, then fill the ListView
                    foreach (ECMInfo e in Response)
                        lstView.Items.Add(ECMListViewItem(e));
                }
            }
            return Response;
        }

        /// <summary>
        /// demonstrates the GetById for the ECMInfoController of the Restful ROADA API
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetResultsById_Click(object sender, EventArgs e)
        {
            // disable the controls in the top panel while processing
            frmMain.EnableControls(splitContainer1.Panel1, false);
            try
            {
                // start by clearing out the old
                lstView.Items.Clear();

                int startId = frmMain.atoi(txtStartId.Text);
                ECMInfo[] Response = null;
                // allows exit from the loop via abort button
                AbortFlag = false;
                btnAbortByRowId.Visible = true;
                do
                {
                    // show a status
                    stsLabel.Text = string.Format("Reading Id: {0}", startId);
                    Application.DoEvents();
                    Response = null;
                    // create the Get string
                    string requestString = string.Format("api/ecminfo?startid={0}&maxrows={1}", startId, txtMaxRowsById.Text);
                    // execute the get
                    Response = PerformECMInfoGet(requestString);
                    // do we have anything?
                    if (Response != null)
                    {
                        // get the next id number to query
                        if (Response.Length > 0)
                            startId = Response[Response.Length - 1].Id;
                        Application.DoEvents();
                    }
                } while (Response != null && Response.Length > 0 && !AbortFlag);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                stsLabel.Text = "Idle";
                // renable the controls in the top panel
                frmMain.EnableControls(splitContainer1.Panel1, true);
                btnAbortByRowId.Visible = false;
            }
        }

        /// <summary>
        /// set the abort flag so that the Get can abort
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAbortByRowId_Click(object sender, EventArgs e)
        {
            // set a flag to get out of the "get" loop
            AbortFlag = true;
        }

        /// <summary>
        /// demonstrates the GetByDate and device id of the Restful ROADA API
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetResultsByDate_Click(object sender, EventArgs e)
        {
            // disable the controls in the top panel while processing
            frmMain.EnableControls(splitContainer1.Panel1, false);
            try
            {
                // start by clearing out the old
                lstView.Items.Clear();

                // create the start/end date from the 2 DateTimePickers
                DateTime startDate = dtpDateStart.Value.Date + dtpTimeStart.Value.TimeOfDay;
                DateTime endDate = dtpDateEnd.Value.Date + dtpTimeEnd.Value.TimeOfDay;
                ECMInfo[] Response = null;
                // allows exit from the loop via abort button
                AbortFlag = false;
                btnAbortByDate.Visible = true;
                do
                {
                    // show a status
                    stsLabel.Text = string.Format("Reading Date: {0}", startDate.ToString(LONG_DATE_STRING));
                    Application.DoEvents();
                    Response = null;
                    // create the Get string
                    // int devId, DateTime startDate, DateTime endDate, int maxRows
                    string requestString = string.Format("api/ecminfo?devid={0}&startdate={1}&enddate={2}&maxrows={3}",
                        txtDevId.Text, startDate.ToString(ISO_DATE_STRING), endDate.ToString(ISO_DATE_STRING), txtMaxRowsById.Text);
                    // execute the get
                    Response = PerformECMInfoGet(requestString);
                    // do we have anything?
                    if (Response != null)
                    {
                        // get the next id number to query
                        if (Response.Length > 0)
                            startDate = Response[Response.Length - 1].UTCDate;
                        Application.DoEvents();
                    }
                } while (Response != null && Response.Length > 0 && !AbortFlag);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                stsLabel.Text = "Idle";
                // renable the controls in the top panel
                frmMain.EnableControls(splitContainer1.Panel1, true);
                btnAbortByDate.Visible = false;
            }
        }

    }
}
